@@ -1,115 +1,10 @@
 #============================================================================
 #============================================================================
-# preparations
 
-#' Check parameter values and save as named vector.
+#' @title Check data for duplicates.
 #'
-#'  Please do not increasing the number of SNPs or individuals beyond the
-#'  numbers present in the datasets, as this may cause R to crash.
-#'
-#' @param GenoM matrix with genotype data, size nInd x nSnp
-#' @param LifeHistData Dataframe with 3 columns:
-#'  \itemize{
-#'  \item{ID: }{max. 30 characters long,}
-#'  \item{Sex: }{1 = females, 2 = males, other numbers = unkown,}
-#'  \item{Birth Year: }{(or hatching year) Zero and negative numbers are
-#'    interpreted as missing values.}}
-#' @param nAgeClasses Number of age classes (= no. rows in AgePriors)
-#' @param MaxSibIter Maximum number of iterations of sibship clustering
-#'   (up to 42).
-#' @param Err Estimated genotyping error rate.
-#' @param MaxMismatch Maximum number of loci at which candidate parent and
-#'  offspring are allowed to be opposite homozygotes, or be excluded.
-#' @param Tfilter Threshold log-likelihood ratio between a proposed
-#'   relationship versus unrelated, to select candidate relatives. Typically a
-#'   negative value, related to the fact that unconditional likelihoods are
-#'   calculated during the filtering steps. More negative values may decrease
-#'   non-assignment, but will increase computational time.
-#' @param Tassign Minimum log-likelihood ratio required for acceptance of
-#'  proposed relationship, relative to next most likely relationship. Higher
-#'  values result in more conservative assignments.
-#' @param MaxSibshipSize  Maximum number of offspring for a single individual
-#'  (a generous safety margin is advised).
-#' @param DummyPrefix character vector of length 2 with prefixes for dummy
-#'   dams (mothers) and sires (fathers); maximum 20 characters each.
-#' @param Complexity Either "full" (default), "simp" (no explicit consideration
-#'   of inbred relationships) or "mono" (monogamous breeding system)
-#' @param FindMaybeRel  Identify pairs of non-assigned likely relatives after
-#'   pedigree reconstruction. Can be time-consuming in large datasets.
-#' @param CalcLLR  Calculate log-likelihood ratios for all assignments. Can be
-#'  time-consuming in large datasets.
-#' @return A 1-row dataframe with parameter values
-
-SeqPrep <- function(GenoM = NULL,
-                     LifeHistData = NULL,
-                     nAgeClasses = 1,
-                     MaxSibIter = 5,
-                     Err = 0.0001,
-                     MaxMismatch = 3,
-                     Tfilter = -2.0,
-                     Tassign = 0.5,
-            				 MaxSibshipSize = 100,
-            				 DummyPrefix = c("F", "M"),
-            				 Complexity = "full",
-            				 FindMaybeRel = TRUE,
-            				 CalcLLR = TRUE)
-
-{
-  if (!is.matrix(GenoM)) stop("'GenoM' should be a numeric matrix")
-  if (!all(GenoM %in% c(0,1,2,-9))) stop("'GenoM' not in <seq> format, please use GenoConvert")
-  nIndG <- nrow(GenoM)
-  IDs_geno <- rownames(GenoM)
-  nSnp <- ncol(GenoM)
-
-  if (Err<0 || Err>1 || !is.double(Err)) stop("invalid value for 'Err'")
-  if (MaxMismatch<0 || !is.wholenumber(MaxMismatch))  stop("invalid value for 'MaxMismatch'")
-  if (!is.double(Tfilter))  stop("invalid value for 'Tfilter'")
-  if (!is.double(Tassign) || Tassign<0)  stop("invalid value for 'Tassign'")
-  if (MaxSibshipSize<0 || !is.wholenumber(MaxSibshipSize)) {
-    stop("invalid value for MaxSibshipSize")
-  }
-  if (!Complexity %in% c("full", "simp", "mono")) stop("invalid value for 'Complexity'")
-  if (!FindMaybeRel %in% c(TRUE, FALSE)) stop("invalid value for 'FindMaybeRel'")
-  if (!CalcLLR %in% c(TRUE, FALSE)) stop("invalid value for 'CalcLLR'")
-  if (MaxSibIter<0 || !is.wholenumber(MaxSibIter))  stop("invalid value for 'MaxSibIter'")
-
-  if (!is.null(LifeHistData)) {
-    nIndLH <- nrow(LifeHistData)
-    if (nIndLH >1 & any(LifeHistData$BY >= 0)) {
-      nAgeClasses <- with(LifeHistData, diff(range(BY[BY >= 0 & ID %in% IDs_geno],
-                                                 na.rm = TRUE))) + 1
-    }
-    if (nAgeClasses > 100) {
-      nAgeClasses <- length(table(LifeHistData$BY[LifeHistData$BY >= 0]))
-       if (nAgeClasses > 100) stop("Cannot handle >100 cohorts!")
-    }
-  }
-
-  Specs <- data.frame(NumberIndivGenotyped = nIndG,
-             NumberSnps = nSnp,
-             GenotypingErrorRate = Err,
-             MaxMismatch = MaxMismatch,
-             Tfilter = Tfilter,
-             Tassign = Tassign,
-      			 nAgeClasses = nAgeClasses,
-             MaxSibshipSize = MaxSibshipSize,
-      			 MaxSibIter = MaxSibIter,
-             DummyPrefixFemale = DummyPrefix[1],
-             DummyPrefixMale = DummyPrefix[2],
-             Complexity = Complexity,
-      			 FindMaybeRel = FindMaybeRel,
-      			 CalcLLR = CalcLLR,
-      			 stringsAsFactors = FALSE)
-  Specs
-}
-
-
-#============================================================================
-#============================================================================
-
-#' Check data for duplicates.
-#'
-#' Check the genotype and life history data for duplicate IDs (not permitted)
+#' @description Check the genotype and life history data for duplicate IDs
+#' (not permitted)
 #' and duplicated genotypes (not advised), and count how many individuals in
 #' the genotype data are not included in the life history data (permitted). The
 #' order of IDs in the genotype and life history data is not required to be
@@ -125,18 +20,18 @@ SeqPrep <- function(GenoM = NULL,
 #'   Please do remove or relabel these to avoid downstream confusion.}
 #' \item{DupGenotype}{Dataframe, duplicated genotypes (with or without
 #'   identical IDs). The specified number of maximum mismatches is allowed,
-#'   and this dataframe may include pairs of closely related individuals.}
+#'   and this dataframe may include pairs of closely related individuals.
+#'   Mismatch = number of SNPs at which genotypes differ, LLR = likelihood
+#'   ratio between 'self' and most likely non-self.}
 #' \item{DupLifeHistID}{Dataframe, rownumbers of duplicated IDs in life
 #'   history data}
 #' \item{NoLH}{Vector, IDs (in genotype data) for which no life history data is
 #'  provided}
 #'
-#' dataframe with the numbers of duplicated genotypes, duplicated IDs in the
-#' genotype data, number of duplicated IDs in the life history data, and number
-#'  of individuals with unknown sex.
-#'
 #' @useDynLib sequoia, .registration = TRUE
 # @useDynLib sequoia duplicates
+#'
+#' @keywords internal
 
 SeqDup <- function(Specs=NULL, GenoM = NULL, LhIN=NULL, quiet=FALSE)
 {
@@ -150,10 +45,12 @@ SeqDup <- function(Specs=NULL, GenoM = NULL, LhIN=NULL, quiet=FALSE)
   DUP <- .Fortran("duplicates",
                   Ng = as.integer(Ng),
                   SpecsInt = as.integer(SpecsInt),
+                  SpecsDbl = as.double(FacToNum(Specs[,"GenotypingErrorRate"])),
                   GenoV = as.integer(GenoV),
                   nDupGenos = as.integer(0),
                   DupGenosFR = integer(2*Ng),  # N x 2 matrix
-                  CntMism = integer(Ng))
+                  CntMism = integer(Ng),
+                  DupLR = double(Ng))
 #                  PACKAGE = "sequoia")
 
   Duplicates <- list()
@@ -170,7 +67,8 @@ SeqDup <- function(Specs=NULL, GenoM = NULL, LhIN=NULL, quiet=FALSE)
                                        row2 = tmp[, 2],
                                        ID1 = gID[tmp[, 1]],
                                        ID2 = gID[tmp[, 2]],
-                                       Mismatch = DUP$CntMism[1:DUP$nDupGenos])
+                                       Mismatch = DUP$CntMism[1:DUP$nDupGenos],
+                                       LLR = DUP$DupLR[1:DUP$nDupGenos])
   }
   if (!is.null(LhIN)) {
     names(LhIN) <- c("ID", "Sex", "BY")
@@ -206,9 +104,10 @@ SeqDup <- function(Specs=NULL, GenoM = NULL, LhIN=NULL, quiet=FALSE)
 #=====================================================================
 #=====================================================================
 
-#' Fortran wrapper
+#' @title Fortran wrapper
 #'
-#' Call Fortran part and convert its output to a list with dataframes.
+#' @description Call Fortran part and convert its output to a list with
+#' dataframes.
 #'
 #' @param ParSib either "par" to call parentage assignment, or "sib" to call
 #'   the rest of the algorithm.
@@ -223,6 +122,7 @@ SeqDup <- function(Specs=NULL, GenoM = NULL, LhIN=NULL, quiet=FALSE)
 #' @return A list with
 #' \item{PedigreePar or Pedigree}{the pedigree}
 #' \item{MaybeParent or MaybeRel}{Non-assigned likely relatives}
+#' \item{MaybeTrio}{Non-assigned parent-parent-offspring trios}
 #' \item{DummyIDs}{Info on dummies}
 #' \item{TotLikParents or TotLikSib}{Total log-likelihood per iteration}
 #'
@@ -230,6 +130,8 @@ SeqDup <- function(Specs=NULL, GenoM = NULL, LhIN=NULL, quiet=FALSE)
 #'
 #' @useDynLib sequoia, .registration = TRUE
 # @useDynLib sequoia makeped deallocall
+#'
+#' @keywords internal
 
 SeqParSib <- function(ParSib = "par",
                       Specs = NULL,
@@ -254,6 +156,7 @@ SeqParSib <- function(ParSib = "par",
     PedPar <- rep(0, Ng*2)
   }
   Complex <- switch(Specs[,"Complexity"], full = 2, simp = 1, mono = 0)
+  UAge <- switch(Specs[,"UseAge"], extra = 2, yes = 1, no = 0)
   PrSb <- switch(ParSib, par = 1, sib = 2)
   nAmbMax <- 7*Ng  # max no. of non-assigned relative pairs to return
 
@@ -263,16 +166,20 @@ SeqParSib <- function(ParSib = "par",
                 MaxMis = FacToNum(Specs[,"MaxMismatch"]),    # 4
                 SMax = SMax,                      # 5
                 nAgeCl = FacToNum(Specs[,"nAgeClasses"]),    # 6
-                Complx = as.integer(Complex),               # 7
+                Complx = as.integer(Complex),                # 7
                 FindMaybe = as.integer(as.logical(Specs[,"FindMaybeRel"])),  # 8
                 CalcLLR = as.integer(as.logical(Specs[,"CalcLLR"])),  # 9
                 quiet = as.integer(quiet),        # 10
-                nAmbMax = as.integer(nAmbMax))       # 11
+                nAmbMax = as.integer(nAmbMax),    # 11
+                UseAge = as.integer(UAge))        # 12
   SpecsDbl <- c(Er = FacToNum(Specs[,"GenotypingErrorRate"]),
                 TF = FacToNum(Specs[,"Tfilter"]),
                 TA = FacToNum(Specs[,"Tassign"]))
-  if (length(as.integer(AgePriors)) != 8*FacToNum(Specs[,"nAgeClasses"])) {
-    stop("'AgePriors' matrix should have size nAgeClasses * 8")
+
+  if (length(as.double(AgePriors)) != 9*FacToNum(Specs[,"nAgeClasses"])) {
+    stop("'AgePriors' matrix should have size nAgeClasses * 9")
+  } else {  # re-arrange columns (for historical reasons...)
+    AP <- AgePriors[, c("MS", "PS", "MGM", "PGF", "MGF", "UA", "M", "P", "FS")]
   }
 
   TMP <- .Fortran("makeped",
@@ -282,7 +189,7 @@ SeqParSib <- function(ParSib = "par",
                   GenoV = as.integer(GenoV),
                   Sex = as.integer(LHL$Sex),
                   BY = as.integer(LHL$BY),
-                  AP = as.double(AgePriors),
+                  AP = as.double(AP),
                   parentsRF = as.integer(PedPar),
                   LrRF = double(3*Ng),
                   OhRF = integer(2*Ng),
@@ -299,7 +206,10 @@ SeqParSib <- function(ParSib = "par",
                   DumBYRF = integer(3*Ng),
                   DumNoff = integer(Ng),
                   DumOff = integer(SMax*Ng),
-                  TotLik = double(42))
+                  TotLik = double(42),
+                  nTrio = as.integer(0),
+                  trioID = integer(3*Ng),
+                  trioLR = double(3*Ng))
 #                  PACKAGE = "sequoia")
 
   TMP$LrRF[abs(TMP$LrRF - 999) < 0.1] <- NA
@@ -310,6 +220,8 @@ SeqParSib <- function(ParSib = "par",
   TMP$DumLrRF <- round(TMP$DumLrRF, 2)
   TMP$OhRF[TMP$OhRF < 0] <- NA
   TMP$AmbigOH[TMP$AmbigOH < 0] <- NA
+  TMP$trioLR[abs(TMP$trioLR - 999) < 0.1] <- NA
+  TMP$trioLR <- round(TMP$trioLR, 2)
 
   DumPfx <- Specs[,c("DummyPrefixFemale", "DummyPrefixMale")]
   dID <- cbind(paste0(DumPfx[1], formatC(1:Ng, width=4, flag=0)),
@@ -340,11 +252,11 @@ SeqParSib <- function(ParSib = "par",
     if (grepl("par", ParSib)) {
       message("assigned ", sum(!is.na(Pedigree$dam)), " dams and ",
            sum(!is.na(Pedigree$sire)), " sires to ", nrow(Pedigree), " individuals")
-   } else {
+    } else {
      message("assigned ", sum(!is.na(Pedigree$dam)), " dams and ",
            sum(!is.na(Pedigree$sire)), " sires to ", Ng, " + ", sum(TMP$Nd),
            " individuals (real + dummy)")
-   }
+    }
   }
 
 
@@ -387,20 +299,38 @@ SeqParSib <- function(ParSib = "par",
       MaybeRel <- herm_unclone_MaybeRel(MaybeRel, Pedigree, LH=LhIN, herm.suf=c("f", "m"))
     }
 
-   if (!quiet && nrow(MaybeRel)>0) {
-     if (grepl("par", ParSib)) {
-     message("there are  ", sum(MaybeRel$TopRel=="PO"),
+    if (!quiet && nrow(MaybeRel)>0) {
+      if (grepl("par", ParSib)) {
+        message("there are  ", sum(MaybeRel$TopRel=="PO"),
              "  likely parent-offspring pairs and ", nrow(MaybeRel)-sum(MaybeRel$TopRel=="PO"),
              " other pairs of likely relatives  which are not assigned, ",
              "perhaps due to unknown birth year(s), please see 'MaybeParent'")
-     } else {
-       message("there are  ", nrow(MaybeRel), "  non-assigned pairs of possible relatives, ",
+      } else {
+        message("there are  ", nrow(MaybeRel), "  non-assigned pairs of possible relatives, ",
                "including  ", sum(MaybeRel$TopRel=="PO"), "  likely parent-offspring pairs; ",
                "please see 'MaybeRel'")
-     }
-  }
+      }
+    }
+    if (nrow(MaybeRel)==0)  MaybeRel <- NULL
   } else  MaybeRel <- NULL
 
+  #=========================
+  # non-assigned parent-parent-offspring trios
+  if (TMP$nTrio>0) {
+    trios <- data.frame(VtoM(TMP$trioID, nr=TMP$nTrio, nc=3),
+                         VtoM(TMP$trioLR, nr=TMP$nTrio, nc=3),
+                         stringsAsFactors=FALSE)
+    names(trios) <- c("id", "parent1", "parent2", "LLRparent1", "LLRparent2", "LLRpair")
+    for (k in 1:3) trios[, k] <- NumToID(trios[, k], k-1, gID, dID)
+
+    if (any(LhIN$Sex==4)) {  # hermaphrodites
+      trios <- herm_unclone_Trios(trios, LH=LhIN, herm.suf=c("f", "m"))
+    }
+
+    if (!quiet) {
+      message("found ", nrow(trios), " parent-parent-offspring trios with parents of unknown sex")
+    }
+  } else  trios <- NULL
 
   #=========================
   # dummies
@@ -433,15 +363,18 @@ SeqParSib <- function(ParSib = "par",
   #=========================
   # output
   if (grepl("par", ParSib)) {
-    return(list(PedigreePar = Pedigree,
+    OUT <- list(PedigreePar = Pedigree,
         MaybeParent = MaybeRel,
-        TotLikParents = TMP$TotLik[1:sum(TMP$TotLik!=0)]))
+        MaybeTrio = trios,
+        TotLikParents = TMP$TotLik[1:sum(TMP$TotLik!=0)])
   } else if (grepl("sib", ParSib)) {
-    return(list(Pedigree = Pedigree,
+    OUT <- list(Pedigree = Pedigree,
                 DummyIDs = DummyIDs,
                 MaybeRel = MaybeRel,
-                TotLikSib = TMP$TotLik[1:sum(TMP$TotLik!=0)]))
+                MaybeTrio = trios,
+                TotLikSib = TMP$TotLik[1:sum(TMP$TotLik!=0)])
   }
+  return(OUT[!sapply(OUT, is.null)])
 }
 
 
@@ -450,9 +383,9 @@ SeqParSib <- function(ParSib = "par",
 
 
 #===============================
-#' Order lifehistory data
+#' @title Order lifehistory data
 #'
-#' Order lifehistory data to match order of IDs in genotype data,
+#' @description Order lifehistory data to match order of IDs in genotype data,
 #' filling in gaps with missing values
 #'
 #' @param LH dataframe with lifehistory information:
@@ -466,6 +399,8 @@ SeqParSib <- function(ParSib = "par",
 #' @return
 #' \item{BY}{Numeric vector with birth years, of same length as gID}
 #' \item{Sex}{Numeric vector with genders, of same length as gID}
+#'
+#' @keywords internal
 
 orderLH <- function(LH=NULL, gID=NULL) {
   if (!is.null(LH)) {
